% This script performs the analysis to help determine the appropriate ARIMA
% (p,d,q) model for the data
% Data can be read in through various means such as .txt, .dat, or .csv.
% This script is an updated version of Harry Hurd's doarima.m
% This script is known to work with R2012a
% Requires: System Identification Toolbox
% Must have pacf.m and acf.m (courtesy of Harry Hurd) in the same folder

%Changable Parameters
ALPHA=.01; % Pr[type 1 error] for threshold line to test acf for N(0,1/sqrt(n)) 
pvec=[0:2]; % orders of AR
minDiffs=0;  % min number of differences
maxDiffs=3; % max number of differences
qvec=[0:2]; % orders of MA
PLOTTS=1; % >0 to plot the time series with acfs
RESIDPLOTS=0; % > 0 to plot residual acfs
RESIDPLOTBEST=0;% > 0 to plot residual acf for best aic
data = importdata('C:\Sync\Box\My Box Files\Documents\MATLAB\TimeSeriesHW1\assign1.txt'); % Put the data path here
dataColumn = 1; % Put the column the data resides in here.
descriptor='Time Series X'; % Description string
xvalues='time'; %Name the x-values




%Program Begins here

totobs = length(data);
dindex= 1:totobs;


for d=minDiffs:maxDiffs
    ddata=data(dindex,dataColumn);             
    for j=1:d      % difference it d times
       ddata=diff(ddata);
    end
    % ddata has been 1st differenced d times
    nd=length(ddata);


    %Create iddata for use with armax command
    %Info: http://www.mathworks.com/help/ident/ref/iddata.html
    id = iddata(ddata,[],1);
    
    if PLOTTS % Plot the time series data
        figure;
        subplot(311);
        plot([1:nd],ddata,'-');
        title(['Result of ',int2str(d),' 1st differences of data: ',descriptor]);

        subplot(312); % now the acf
        rhod=xcov(ddata,'coeff'); % this returns the acf normalized by the variance
        % it is even, so ignore the negative lags and take only half of the positive ones
        rhod=rhod(nd:nd+floor(nd/2)); % floor needed in case nd/2 not integer
        nrho=length(rhod);
        stem(rhod,'.');	% a stem plot with dot marker
        hold on;
        plot(zeros(1,nrho));
        thr=norminv(1-ALPHA/2,0,1/sqrt(nd));
        plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
        title(['ACF, n=',int2str(nd),' alpha = ',num2str(ALPHA)]);
        hold off;
        
        subplot(313)
        pacfPoints = pacf(ddata,nrho,0);
        stem(pacfPoints,'.');
        hold on;
        plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
        title(['PACF, nd=',int2str(nd),' alpha = ',num2str(ALPHA)]);
        hold off;
        
    end

    % now estimate ARMA model; ARMAX is AX=BU + Ce so identify phi with A and theta with C
    np=length(pvec);
    nq=length(qvec);
    aicsave=-99*ones(np,nq);
    fpesave=-99*ones(np,nq);
    minaic=1e+6;
    for pp=1:np 
        p=pvec(pp);
        for qq=1:nq
            q=qvec(qq);
            if p+q ~=0
                orders=[p q];
                m=armax(id,orders);      % m is a structure with the model stuff in it
                resids=pe(m,id);         % pe returns the prediction errors

                % Get the data out of the time series object
                resids = resids.OutputData;



                nres=length(resids);
                rhores=xcov(resids,'coeff'); % this returns the acf normalized by the variance
                % it is even, so ignore the negative lags and take only half of the positive ones

                rhores=rhores(nres:nres+floor(nres/2)); % floor needed in case nd/2 not integer
                nrho=length(rhores);                    % now rhores(1) is zero lag
                % next compute the Ljung - Box statistic and P-values
                deltak=floor(nrho/10)+1;
                kvec=[p+q+1:deltak:p+q+1+4*deltak];
                for kk=1:5
                    Q=0;
                    for j=2:kvec(kk)+1
                        Q=Q+(rhores(j).^2)/(nd-j);
                    end
                    Q=nd*(nd-1)*Q;
                    ljpv(kk)=1-chi2cdf(Q,kvec(kk)-p-q);  % df=kvec(kk)-p-q
                end
                aicsave(pp,qq)=aic(m);
                fpesave(pp,qq)=fpe(m);
                if aicsave(pp,qq) < minaic
                    minaic=aicsave(pp,qq); % save the min
                    pbest=p;
                    qbest=q;
                    mbest=m;
                    rhoresbest=rhores;
                    residsbest=resids;
                end
                disp(sprintf('Number of Differences : %i',d));
                disp(sprintf('(p,d,q)=(%d,%d,%d) aic=%.5f  fpe=%.5f',p,d,q,aicsave(pp,qq),fpesave(pp,qq)));
                QQ=[kvec;ljpv];
                disp(sprintf('Ljung-Box P-values : '));
                disp(sprintf('  K=%d P-v=%.3f \n',QQ(:)));

                %Show all coefficients
                [A,B,C,D,F]=polydata(m);
                disp(['\phi coefficients : ',num2str(A)]); 
                disp(['\theta coefficients : ',num2str(C)]);
                disp(sprintf('\n'));

                if RESIDPLOTS
                %%%%%%%%%% resids and acf
                figure;
                subplot(311);
                plot([1:nd],resids,'-');
                title(['Residuals from ARIMA(',sprintf('%d,%d,%d)',p,d,q),' for data: ',descriptor]);

                subplot(312); % now the acf

                stem(rhores,'.');	% a stem plot with dot marker
                hold on;
                plot(zeros(1,nrho));
                thr=norminv(1-ALPHA/2,0,1/sqrt(nres));
                plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
                title(['ACF of residuals, n=',int2str(nres),' alpha = ',num2str(ALPHA)]);
                
                subplot(313)
                pacfPoints = pacf(resids,nrho,0);
                stem(pacfPoints,'.');
                hold on;
                plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
                title(['PACF of residuals, nd=',int2str(nd),' alpha = ',num2str(ALPHA)]);
                hold off;
                
                end % RESIDPLOTS

            end
        end
    end
    disp(['SUMMARY OF ARIMA MODELS FOR DATA : ',descriptor]);
    disp(sprintf('Number of Differences : %i',d));
    disp(sprintf('AIC min at (p,d,q)=(%d,%d,%d) aic=%.5f',pbest,d,qbest,aic(mbest)));
    [A,B,C,D,F]=polydata(mbest);
    disp(['\phi coefficients : ',num2str(A)]);
    disp(['\theta coefficients : ',num2str(C)]);
    disp(sprintf('\n'));

                if RESIDPLOTBEST
                %%%%%%%%%% resids and acf
                figure;
                subplot(311);
                plot([1:nd],residsbest,'-');
                title(['Residuals from ARIMA(',sprintf('%d,%d,%d)',pbest,d,qbest),' for data: ',descriptor]);

                subplot(312); % now the acf

                stem(rhoresbest,'.');	% a stem plot with dot marker
                hold on;
                plot(zeros(1,nrho));
                thr=norminv(1-ALPHA/2,0,1/sqrt(nres));
                plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
                title(['ACF of residuals, n=',int2str(nres),' alpha = ',num2str(ALPHA)]);
                
                subplot(313)
                pacfPoints = pacf(residsbest,nrho,0);
                stem(pacfPoints,'.');
                hold on;
                plot([1:nrho],thr*ones(1,nrho),'--r',[1:nrho],-thr*ones(1,nrho),'--r');
                title(['PACF of residuals, nd=',int2str(nd),' alpha = ',num2str(ALPHA)]);
                hold off;
                end % RESIDPLOTS


    % now plot periodogram and estimated spectra from model

    %%%%%%%%%%%%%%  plot |W(\lambda)|^2
    NLAM=100;	% no. pts to plot in  [0,2\pi]
    dlam=pi/NLAM;
    lams=[0:dlam:pi-dlam];
    clear i;
    e=exp(-i*lams);

    thetaspec=polyval(C,e);
    phispec=polyval(A,e);
    sigr=std(resids);

    W=thetaspec./phispec;
    %%% xt=newpgram(x,nfft,np1,np2,halflen,alpha,rejalpha,plsw,logsw,dataname)
    %estspec=pgram(ddata,nd,[],0,.01,descriptor);
    %estspec=newpgram(ddata,NLAM*2,1,NLAM,8,.01,.001,0,0,descriptor);
    %estspec=estspec(1:floor(nd/2));

    %estspec=avspec(ddata',NLAM*2,NLAM,0); % 
    modspec=(sigr^2)/(2*pi)*abs(W).^2;

    %figure;
    %semilogy(lams);
    %xlabel(' 0 \leq \lambda < \pi');
    %v=axis;
    %text(v(1)+.2*(v(2)-v(1)),v(3)+.8*(v(4)-v(3)),['theta:',...
    %      sprintf(' %4.2f',C),sprintf('\n phi:'),sprintf(' %4.2f',A)]);
    %title('spectral density estimates - log scale');
    %legend('periodogram','estimated model');
end
